﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using Autodesk.AutoCAD.Windows;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Ribbon;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.BoundaryRepresentation;
using Autodesk.AutoCAD.GraphicsInterface;
using BrFace = Autodesk.AutoCAD.BoundaryRepresentation.Face;

[assembly: ExtensionApplication(typeof(MotionAddon.ACMMainClass))]

namespace MotionAddon
{
    class ACMMainClass : IExtensionApplication
    {
        public void Initialize()
        {
            Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;
            ed.WriteMessage("\ntesting 123\n");
            aACMMenu.CheckCUIMenu();
            aACMMenu.CreateACMContextMenu();
            aACMInterface.CreateACMExplorer();
            aACMRegistration.initializeRegister();
        }
        public void Terminate()
        {
        }

        static public ACMInterface aACMInterface = new ACMInterface();
        static public ACMRegistration aACMRegistration = new ACMRegistration();
        static public ACMMenu aACMMenu = new ACMMenu();
        static public ACMPart SelectedPart = new ACMPart();
        static public Boolean SketchMode = false;
        //event class
        public class ACMEvent
        {
            public ACMEvent(Database db)
            {
                this._db = db;
                _db.ObjectAppended += new ObjectEventHandler(OnObjectAppended);
                //_db.ObjectModified += new ObjectEventHandler(OnObjectModified);
            }

            ~ACMEvent()
            {
                try
                {
                    _db.ObjectAppended -= new ObjectEventHandler(OnObjectAppended);
                    //_db.ObjectModified -= new ObjectEventHandler(OnObjectModified);
                }
                catch (System.Exception)
                {
                }
            }

            private Database _db;
            public Database setDataBase 
            {
                set { _db = value; }
                get { return _db; }
            }

            void OnObjectAppended(object sender, ObjectEventArgs e)
            {
                
                try
                {
                    if (SketchMode == true)
                    {
                        if (SelectedPart != null)
                        {
                            ACMExplorer M_E = ACMInterface.aACMExplorer as ACMExplorer;
                            Database db = (Database)sender;
                            if (db != null)
                            {
                                //e is the ObjectEventArgs
                                if ((e.DBObject.GetType() == typeof(Solid3d))) 
                                //check if the event is triggered by object of type Solid3d
                                {
                                    M_E.AddPartObjectID(SelectedPart, e.DBObject.ObjectId);
                                    SketchMode = false;
                                    M_E.UpdateMotionView();
                                    ////////////////
                                    using (Transaction trans = db.TransactionManager.StartTransaction())
                                    {
                                        Solid3d newSolid = (Solid3d)trans.GetObject(e.DBObject.ObjectId, OpenMode.ForRead);
                                        Brep newBrep = new Brep(newSolid);
                                        
                                        foreach (Complex aComplex in newBrep.Complexes)
                                        {
                                            foreach (Shell aShell in aComplex.Shells)
                                            {
                                                foreach (BrFace aBrFace in aShell.Faces)
                                                {

                                                }
                                            }

                                        }
                                        FullSubentityPath paths = newBrep.SubentityPath;
                                        ObjectId[] newObjectId = paths.GetObjectIds();

                                    }
                                }
                            }
                        }
                    }
                }
                catch
                {
                }
           

            }

            void OnObjectModified(object sender, ObjectEventArgs e)
            {
     
                Database db = (Database)sender;
                Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;
                if (e.DBObject.GetType() == typeof(Solid3d))
                {
                    Solid3d newSolid = (Solid3d)e.DBObject;
                    try
                    {
                        ResultBuffer buf = newSolid.XData;
                        if (buf != null)
                        {
                            foreach (TypedValue TV in buf)
                            {
                                if (TV.TypeCode == (int)DxfCode.ExtendedDataHandle)
                                {
                                    long lg = System.Convert.ToInt64(TV.Value.ToString(), 16);
                                    Handle hd = new Handle(lg);
                                    ObjectId id = db.GetObjectId(false, hd, -1);
                                    using (Transaction Trans = db.TransactionManager.StartTransaction())
                                    {
                                        BlockReference Markerbr = (BlockReference)Trans.GetObject(id, OpenMode.ForRead);
                                        ResultBuffer MarkerData = Markerbr.XData;
                                        int count = 0;
                                        double[] resultdouble = new double[3];
                                        foreach (TypedValue TVMarker in MarkerData)
                                        {
                                            if (TVMarker.TypeCode == (int)DxfCode.ExtendedDataAsciiString)
                                            {
                                                resultdouble[count] = System.Convert.ToDouble(TVMarker.Value.ToString());
                                                count++;
                                            }
                                        }
                                        if ((count == 3) && (resultdouble != null))
                                        {
                                            Vector3d vector_stored = new Vector3d(resultdouble);
                                            Point3d solidpoint = newSolid.MassProperties.Centroid;
                                            Point3d markerpoint = Markerbr.Position;
                                            Vector3d vector_current = solidpoint - markerpoint;
                                            ed.WriteMessage("\nCurrent :" + vector_current.ToString());
                                            ed.WriteMessage("\nStored :" + vector_stored.ToString());
                                            ed.WriteMessage("\nLength strored : " + vector_stored.Length.ToString());
                                            ed.WriteMessage("\nLength Current : " + vector_current.Length.ToString());
                                            Vector3d apart = vector_current - vector_stored;
                                            ed.WriteMessage("\nApart :" + apart.ToString());
                                            double apartX = vector_current.X - vector_stored.X;
                                            double apartY = vector_current.Y - vector_stored.Y;
                                            double apartZ = vector_current.Z - vector_stored.Z;
                                            if ((apart.Length < 0.00001 || apart.Length > 0.00001) && apart.Length != 0.0)
                                            {
                                                DocumentLock docLock = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.LockDocument();
                                                Markerbr.UpgradeOpen();
                                                Markerbr.TransformBy(Matrix3d.Displacement(apart));
                                                Trans.Commit();
                                                docLock.Dispose();
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    catch
                    {
                    }
                }
            }
        }
    }
}


