﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Win32;
using System.Reflection;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;

namespace MotionAddon
{
    public class ACMRegistration
    {
        public ACMRegistration()
        {
            sProductKey = HostApplicationServices.Current.RegistryProductRootKey;
            sApplicationName = "ACADMotion";
        }

        private string sProductKey;
        private string sApplicationName;
        private string _LoadOption;

        public void initializeRegister()
        {
            if (!ACMRegistered()) 
            { 

                //RegisterACM(); 
            }
            if (ACMRegistered())
            {
                if (LoadOption == "true")
                {
                    ACMMainClass.aACMInterface.CreateRegistrationDialog();
                }
            }

        }
        public string LoadOption
        {
            set 
            {
                try
                {
                    string sLoadOption = value;
                    RegistryKey regAcadProdKey = Registry.CurrentUser.OpenSubKey(sProductKey);
                    RegistryKey regAcadAppKey = regAcadProdKey.OpenSubKey("Applications", true);
                    using (RegistryKey ACMKey = regAcadAppKey.OpenSubKey(sApplicationName, true))
                    {
                        if (ACMKey.GetValue("LOADREGDIALOG") != null)
                        {
                            ACMKey.DeleteValue("LOADREGDIALOG");
                        }
                        ACMKey.SetValue("LOADREGDIALOG", sLoadOption);
                    }
                    regAcadAppKey.Close();
                }
                catch
                {
                }
            }
            get 
            {
                try
                {
                    RegistryKey regAcadProdKey = Registry.CurrentUser.OpenSubKey(sProductKey);
                    RegistryKey regAcadAppKey = regAcadProdKey.OpenSubKey("Applications", true);
                    using (RegistryKey ACMKey = regAcadAppKey.OpenSubKey(sApplicationName, true))
                    {
                        if (ACMKey.GetValue("LOADREGDIALOG") != null)
                        {
                            _LoadOption = (string)ACMKey.GetValue("LOADREGDIALOG");
                        }
                    }
                    regAcadAppKey.Close();
                }
                catch
                {
                }
                return _LoadOption;
            }
        }
        public Boolean ACMRegistered()
        {
            RegistryKey regAcadProdKey = Registry.CurrentUser.OpenSubKey(sProductKey);
            RegistryKey regAcadAppKey = regAcadProdKey.OpenSubKey("Applications", true);

            // Check if "ACADMotion" exists
            string[] subKeys = regAcadAppKey.GetSubKeyNames();
            foreach (string subKey in subKeys)
            {
                // If the application is already registered, exit
                if (subKey.Equals(sApplicationName))
                {
                    regAcadAppKey.Close();
                    return true;
                }
            }
            regAcadAppKey.Close();
            return false;
        }
        public void RegisterACM()
        {
            try
            {
                RegistryKey regAcadProdKey = Registry.CurrentUser.OpenSubKey(sProductKey);
                RegistryKey regAcadAppKey = regAcadProdKey.OpenSubKey("Applications", true);

                string sAssemblyPath = Assembly.GetExecutingAssembly().Location;

                // Register the application
                using (RegistryKey regAppAddInKey = regAcadAppKey.CreateSubKey(sApplicationName))
                {
                    regAppAddInKey.SetValue("DESCRIPTION", sApplicationName, RegistryValueKind.String);
                    regAppAddInKey.SetValue("LOADCTRLS", 2, RegistryValueKind.DWord);
                    regAppAddInKey.SetValue("LOADER", sAssemblyPath, RegistryValueKind.String);
                    regAppAddInKey.SetValue("MANAGED", 1, RegistryValueKind.DWord);
                    regAppAddInKey.SetValue("LOADREGDIALOG", "true");
                }
                regAcadAppKey.Close();
            }
            catch
            {
            }
        }
        public void UnRegisterACM()
        {
            RegistryKey regAcadProdKey = Registry.CurrentUser.OpenSubKey(sProductKey);
            RegistryKey regAcadAppKey = regAcadProdKey.OpenSubKey("Applications", true);

            // Delete the key for the application
            regAcadAppKey.DeleteSubKeyTree(sApplicationName);
            regAcadAppKey.Close();
        }

        public void RegACMDwgApp()
        {
            try
            {
                //Get the database and start a transaction
                Database db = Application.DocumentManager.MdiActiveDocument.Database;
                using (Transaction Trans = db.TransactionManager.StartTransaction())
                {
                    string regAppName = "ACADMotion";
                    //Get the Application Table 
                    RegAppTable XdTbl = (RegAppTable)Trans.GetObject(db.RegAppTableId, OpenMode.ForWrite);
                    //check if ACADMotion already existed
                    if (!XdTbl.Has(regAppName))
                    {
                        //Create a new Application Table Record and add it to the Application Table
                        RegAppTableRecord xdRec = new RegAppTableRecord();
                        xdRec.Name = regAppName;
                        XdTbl.Add(xdRec);
                        Trans.AddNewlyCreatedDBObject(xdRec, true);
                    }
                    Trans.Commit();
                }
            }
            catch
            {
            }
        }
    }
}
