﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Colors;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.Runtime;
using INMotion2010a;

using System.Timers;

namespace MotionAddon
{
    public partial class ACMExplorer : UserControl
    {
        //constructor
        public ACMExplorer()
        {
            InitializeComponent();
            /////////////////////////////////////////
            ////////// The only COM CADSM ///////////
            
            /////////////////////////////////////////
            this.PreviousNode = null;
            this.PreviousPartID = ObjectId.Null;
            this.PreviousPartColor = null;
            this.PartsArray = new ArrayList();
            this.MovingPartsArray = new ArrayList();
            this.MarkersArray = new ArrayList();
            this.SelectedACMPart = new ACMPart();
            this.SelectedACMMarker = new ACMMarker();
            this.SimulationStatus = false;
            aTimer.Interval = 1000;
            aTimer.Elapsed += new System.Timers.ElapsedEventHandler(movepart);
        }

        //private variable
        private TreeNode PreviousNode;
        private ObjectId PreviousPartID;
        private Autodesk.AutoCAD.Colors.Color PreviousPartColor;
        private string AssemblyDocName;
        private ACMPart SelectedACMPart;
        private ACMMarker SelectedACMMarker;
        private Autodesk.AutoCAD.Colors.Color aHighlightColor = Autodesk.AutoCAD.Colors.Color.FromColorIndex(ColorMethod.ByAci, 1);
        //public variable
        static public ACMAssembly aACMAssembly; //main assembly
        static public INMotion2010aClass COMCADSM = new INMotion2010aClass();
        static public Document ProjectDocument;
        public ArrayList PartsArray;
        public ArrayList MovingPartsArray;
        public ArrayList MarkersArray;
        public Boolean SimulationStatus;
        public static System.Timers.Timer aTimer = new System.Timers.Timer();
        public static short Current_Frame = 1;
        public static short TotalNoOfFrame;

        //Initialise Treenode
        public void InitialiseTreeNode()
        {
            TreeNode MAssembly = new TreeNode(AssemblyDocName);
            System.Windows.Forms.TreeNode Parts = new System.Windows.Forms.TreeNode("Parts");
            System.Windows.Forms.TreeNode Markers = new System.Windows.Forms.TreeNode("Markers");
            System.Windows.Forms.TreeNode Joints = new System.Windows.Forms.TreeNode("Joints");
            System.Windows.Forms.TreeNode Constraints = new System.Windows.Forms.TreeNode("Constraints");
            MAssembly.ForeColor = System.Drawing.Color.Gray;
            MAssembly.Name = "Assembly";
            Parts.NodeFont = new Font(ACMTreeView.Font, FontStyle.Bold);
            Parts.Name = "Parts";
            Markers.NodeFont = new Font(ACMTreeView.Font, FontStyle.Bold);
            Markers.Name = "Markers";
            Joints.NodeFont = new Font(ACMTreeView.Font, FontStyle.Bold);
            Joints.Name = "Joints";
            ACMTreeView.Nodes.Add(MAssembly);
            ACMTreeView.Nodes["Assembly"].Nodes.Add(Parts);
            ACMTreeView.Nodes["Assembly"].Nodes.Add(Markers);
            ACMTreeView.Nodes["Assembly"].Nodes.Add(Joints);
            ACMTreeView.Nodes["Assembly"].Nodes.Add("Setting", "Setting");
            ACMTreeView.Nodes["Assembly"].Nodes["Setting"].Nodes.Add("Simulation", "Simulation");
            ACMTreeView.Nodes["Assembly"].Nodes["Setting"].Nodes.Add("Playback", "Playback");
            ACMTreeView.Nodes["Assembly"].Nodes["Setting"].Nodes.Add("Gravity", "Gravity");
            ACMTreeView.ExpandAll();
        }

        //update treeview
        public void UpdateMotionView()
        {
            ACMTreeView.Nodes.Clear();
            InitialiseTreeNode();
            foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
            {
                //Add node to the parent node "Parts"
                ACMTreeView.Nodes["Assembly"].Nodes["Parts"].Nodes.Add(aACMPart.Name, aACMPart.Name);

                //Add node "Marker"
                ACMTreeView.Nodes["Assembly"].Nodes["Parts"].Nodes[aACMPart.Name].Nodes.Add("Marker", "Marker");
                ACMTreeView.Nodes["Assembly"].Nodes["Parts"].Nodes[aACMPart.Name].Nodes["Marker"].ForeColor = System.Drawing.Color.Gray;
                foreach (ACMMarker aACMMarker in aACMPart.cACMMarker)
                {
                    ACMTreeView.Nodes["Assembly"].Nodes["Parts"].Nodes[aACMPart.Name].Nodes["Marker"].Nodes.Add(aACMMarker.Name,aACMMarker.Name);
                }
            }
            foreach (classMovingPart newMovingPart in MovingPartsArray)
            {
                ACMTreeView.Nodes["Assembly"].Nodes["Moving Parts"].Nodes.Add(new TreeNode(newMovingPart.ObjectName));
            }
        }

        //show menu on right click
        private void ACMTreeView_MouseClick(object sender, MouseEventArgs e)
        {
            try
            {
                if (e.Button == MouseButtons.Right)
                {
                    Point atPoint = new Point(e.X, e.Y);
                    TreeNode aTreeNode = ACMTreeView.GetNodeAt(atPoint);
                    if (aTreeNode != null) { ACMTreeView.SelectedNode = aTreeNode; }
                    if (ACMTreeView.SelectedNode == ACMTreeView.Nodes["Assembly"])
                    {
                        AssemblyMenu.Show(ACMTreeView, atPoint);
                    }
                    if (ACMTreeView.SelectedNode.Parent == ACMTreeView.Nodes["Assembly"].Nodes["Parts"])
                    {
                        PartMenu.Show(ACMTreeView, atPoint);
                    }
                    if (ACMTreeView.SelectedNode.Parent.Name == "Markers")
                    {
                        MarkerMenu.Show(ACMTreeView, atPoint);
                    }
                    if (ACMTreeView.SelectedNode == ACMTreeView.Nodes["Assembly"].Nodes["Setting"].Nodes["Simulation"])
                    {
                        Setting.Show(ACMTreeView, atPoint);
                    }
                }
            }
            catch
            {
            }
        }

        //mouse over
        private void ACMTreeView_NodeMouseHover(object sender, TreeNodeMouseHoverEventArgs e)
        {
            if (e.Node.Parent == ACMTreeView.Nodes["Assembly"].Nodes["Parts"])
            {
                if (PreviousNode == null)
                {
                    PreviousNode = e.Node;
                    e.Node.NodeFont = new Font(ACMTreeView.Font, FontStyle.Bold);
                    ACMPart aACMPart = SearchIncACMPart(e.Node.Text);
                    PreviousPartID = aACMPart.ObjectID;
                    PreviousPartColor = GetPartColor(aACMPart.ObjectID);
                    using (DocumentLock docLock = ProjectDocument.LockDocument())
                    {
                        HighlightObject(aACMPart.ObjectID, Autodesk.AutoCAD.Colors.Color.FromColorIndex(ColorMethod.ByAci, 1));
                    }
                }
                if ((e.Node != PreviousNode) && (PreviousNode != null) && (PreviousPartID != null) && (PreviousPartColor != null))
                {
                    PreviousNode.NodeFont = new Font(ACMTreeView.Font, FontStyle.Regular);
                    e.Node.NodeFont = new Font(ACMTreeView.Font, FontStyle.Bold);
                    ACMPart newPart = SearchIncACMPart(e.Node.Text);
                    using (DocumentLock docLock = ProjectDocument.LockDocument())
                    {
                        HighlightObject(PreviousPartID, PreviousPartColor);
                        PreviousPartColor = GetPartColor(newPart.ObjectID);
                        HighlightObject(newPart.ObjectID, aHighlightColor);
                    }
                    PreviousNode = e.Node;
                    PreviousPartID = newPart.ObjectID;
                }
            }
            else
            {
                if (PreviousNode != null)
                {
                    DocumentLock docLock = ProjectDocument.LockDocument();
                    HighlightObject(PreviousPartID, PreviousPartColor);
                    docLock.Dispose();
                    PreviousNode.NodeFont = new Font(ACMTreeView.Font, FontStyle.Regular);
                    PreviousNode = null;
                    PreviousPartColor = null;
                }
            }
        }

        //mouse leave
        private void ACMTreeView_MouseLeave(object sender, EventArgs e)
        {
            if (PreviousNode != null)
            {
                DocumentLock docLock = ProjectDocument.LockDocument();
                HighlightObject(PreviousPartID, PreviousPartColor);
                docLock.Dispose();
                PreviousNode.NodeFont = new Font(ACMTreeView.Font, FontStyle.Regular);
                PreviousNode = null;
                PreviousPartColor = null;
            }
        }

        //Menu Item Click
        private void Setting_MouseClick(object sender, MouseEventArgs e)
        {
            ACMInterface interfaceobject = ACMMainClass.aACMInterface;
            if (e.Button == MouseButtons.Left){ interfaceobject.CreateSimulationSettingDialog(); }
        }

        private void NewProject_Click(object sender, EventArgs e)
        {
            ProjectDocument = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument;
            string tempAssemblyDocName = ProjectDocument.Name.ToString();
            int AssemblyLength = tempAssemblyDocName.Length;
            if ((tempAssemblyDocName[AssemblyLength - 4] == '.') && (tempAssemblyDocName[AssemblyLength - 3] == 'd') && (tempAssemblyDocName[AssemblyLength - 2] == 'w') && (tempAssemblyDocName[AssemblyLength - 1] == 'g'))
            {
                AssemblyDocName = tempAssemblyDocName.Remove(AssemblyLength - 4, 4);
            }
            AddACMAssembly();
        }

        private void Simulator_Click(object sender, EventArgs e)
        {
            ACMMainClass.aACMInterface.CreateSimulatorDialog();
        }

        private void NewPart_Click(object sender, EventArgs e)
        {
            ACMMainClass.aACMInterface.CreateAddPartDialog();
        }
        
        //Assembly menu item
        private void ASMmarkerMenuItem_Click(object sender, EventArgs e)
        {
            ACMMainClass.aACMInterface.CreateAddASMMarkerDialog();
        }


        //part menu item
        private void NewMarker_Click(object sender, EventArgs e)
        {
            if (ACMTreeView.SelectedNode.Parent == ACMTreeView.Nodes["Assembly"].Nodes["Parts"])
            {
                ACMPart aACMPart = SearchIncACMPart(ACMTreeView.SelectedNode.Text);
                if (aACMPart != null)
                {
                    SelectedACMPart = aACMPart;
                    ACMMainClass.aACMInterface.CreateAddMarkerDialog();
                }
            }
        }

        private void revoluteJointToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ACMInterface interfaceobject = ACMMainClass.aACMInterface;
            interfaceobject.CreateInsertNewRevoluteJointDialog();
        }

        private void sketchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ACMTreeView.SelectedNode.Parent == ACMTreeView.Nodes["Assembly"].Nodes["Parts"])
            {
                ACMPart NewPart = SearchIncACMPart(ACMTreeView.SelectedNode.Text);
                if (NewPart.ObjectID == ObjectId.Null)
                {
                    ACMMainClass.SketchMode = true;
                    ACMMainClass.SelectedPart = NewPart;
                }
            }

        }

        private void jointToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ACMMainClass.aACMInterface.CreateAddJointDialog();
        }

        //marker menu item
        private void positionAngleMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                ACMMarker aACMMarker = SearchIncACMMarker(ACMTreeView.SelectedNode.Text);
                if (aACMMarker != null)
                {
                    SelectedACMMarker = aACMMarker;
                    ACMMainClass.aACMInterface.CreateEditMarkerDialog();
                }
            }
            catch
            {
            }
        }

        ///////////////////////////////////////////////////////////////////////////
        /////////////////// end 

        public bool InPartsArraybyID(ObjectId ID)
        {
            foreach (ACMPart newclass in PartsArray)
            {
                if (ID == newclass.ObjectID) return true;
            }
            return false;
        }

        public bool InPartsArraybyName(string PartName)
        {
            foreach (ACMPart newclass in PartsArray)
            {
                if (PartName == newclass.Name) return true;
            }
            return false;
        }

        private ACMPart SearchIncACMPart(string PartName)
        {
            ACMPart returnPart = new ACMPart();
            foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
            {
                if (aACMPart.Name == PartName)
                {
                    returnPart = aACMPart;
                }
            }
            return returnPart;
        }

        private ACMMarker SearchIncACMMarker(string MarkerName)
        {
            ACMMarker returnMarker = new ACMMarker();
            foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
            {
                foreach (ACMMarker aACMMarker in aACMPart.cACMMarker)
                {
                    if (aACMMarker.Name == MarkerName)
                    {
                        returnMarker = aACMMarker;
                    }
                }
            }
            return returnMarker;
        }

        private bool InMovingPartArraybyName(string MovingPartName)
        {
            foreach (classMovingPart newclass in MovingPartsArray)
            {
                if (MovingPartName == newclass.ObjectName) return true;
            }
            return false;
        }

        private bool InMovingPartArraybyID(ObjectId ID)
        {
            foreach (classMovingPart newclass in MovingPartsArray)
            {
                if (ID == newclass.ObjectID) return true;
            }
            return false;
        }

        private classMovingPart SearchMovingPartInArray(string MovingPartName)
        {
            classMovingPart returnMovingPart = new classMovingPart();
            foreach (classMovingPart newMovingPart in MovingPartsArray)
            {
                if (newMovingPart.ObjectName == MovingPartName){ returnMovingPart = newMovingPart; }
            }
            return returnMovingPart;
        }

        public void HighlightObject(ObjectId ID ,Autodesk.AutoCAD.Colors.Color HighlightColor)
        {
            if (ProjectDocument != null)
            {
                Database db = ProjectDocument.Database;
                using (Transaction Trans = db.TransactionManager.StartTransaction())
                {
                    try
                    {
                        BlockTable bt = (BlockTable)Trans.GetObject(db.BlockTableId, OpenMode.ForRead);
                        BlockTableRecord btr = (BlockTableRecord)Trans.GetObject(db.CurrentSpaceId, OpenMode.ForWrite);

                        Entity aEntity = (Entity)Trans.GetObject(ID, OpenMode.ForWrite);
                        if (aEntity != null)
                        {
                            if (aEntity.GetType() == typeof(BlockReference))
                            {
                                BlockReference aBlockReference = (BlockReference)aEntity;
                                
                                
                            }
                            if (aEntity.GetType() == typeof(Solid3d))
                            {
                                aEntity.Color = HighlightColor;
                                Trans.Commit();
                            }
                        }
                        aEntity.Dispose();
                    }
                    catch
                    {
                    }
                }
            }
        }

        private Autodesk.AutoCAD.Colors.Color GetPartColor(ObjectId ID)
        {
            Autodesk.AutoCAD.Colors.Color returnColor = new Autodesk.AutoCAD.Colors.Color();
            Database db = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Database;
            using (Transaction Trans = db.TransactionManager.StartTransaction())
            {
                try
                {
                    BlockTable bt = (BlockTable)Trans.GetObject(db.BlockTableId, OpenMode.ForRead);
                    BlockTableRecord btr = (BlockTableRecord)Trans.GetObject(db.CurrentSpaceId, OpenMode.ForRead);

                    Solid3d ent;
                    ent = Trans.GetObject(ID, OpenMode.ForRead) as Solid3d;
                    if (ent != null){ returnColor = ent.Color; }
                    ent.Dispose();
                }
                catch
                {
                }
            }
            return returnColor;
        }

        public void AddPartObjectID(ACMPart NewPart, ObjectId ID)
        {
            try
            {
                int tempindex = PartsArray.IndexOf(NewPart);
                NewPart.ObjectID = ID;
                PartsArray[tempindex] = NewPart;
            }
            catch
            {
            }
        }

        public void AddRotation(int AngleOfRotation, Vector3d AxisOfRotation, Point3d BasePoint)
        {
            try
            {
                if (!InMovingPartArraybyName(ACMTreeView.SelectedNode.Text))
                {
                    ACMPart newPart = SearchIncACMPart(ACMTreeView.SelectedNode.Text);
                    MovingPartsArray.Add(new classMovingPart(newPart.Name, newPart.ObjectID, AngleOfRotation, AxisOfRotation, BasePoint));
                    UpdateMotionView();
                }
                else
                {
                    if (InMovingPartArraybyName(ACMTreeView.SelectedNode.Text))
                    {
                        classMovingPart newMovingPart = SearchMovingPartInArray(ACMTreeView.SelectedNode.Text);
                        int currentLocation = MovingPartsArray.IndexOf(newMovingPart);
                        MovingPartsArray.Remove(newMovingPart);
                        newMovingPart.AngleOfRotation = AngleOfRotation;
                        newMovingPart.AxisofRotation = AxisOfRotation;
                        newMovingPart.BasePoint = BasePoint;
                        MovingPartsArray.Insert(currentLocation, newMovingPart);
                    }
                }
            }
            catch
            {
            }
        }

        //create marker
        public Point3d selectpoint() 
        {
            Editor ed = ProjectDocument.Editor; 
            Point3d _point = new Point3d();
            try
            {
                PromptPointOptions prPointOptions = new PromptPointOptions("");
                PromptPointResult prPointRes;
                prPointRes = ed.GetPoint(prPointOptions);
                if (prPointRes.Status != PromptStatus.OK)
                {
                    _point = new Point3d(0, 0, 0);
                }
                else
                {
                    _point = prPointRes.Value;
                }
            }
            catch 
            {
            }
            return _point;
        }
        public ObjectId selectobjects()
        {
            ObjectId aObjectId = new ObjectId();
            using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
            {
                // Request for objects to be selected in the drawing area
                PromptSelectionResult PromptResult = ProjectDocument.Editor.GetSelection();
                if (PromptResult.Status == PromptStatus.OK)
                {
                    SelectionSet SSet = PromptResult.Value;
                    foreach (SelectedObject aSelectedObject in SSet)
                    {
                        if (aSelectedObject != null)
                        {
                            Entity aEntity = (Entity)Trans.GetObject(aSelectedObject.ObjectId, OpenMode.ForRead);
                            if (aEntity.GetType() == typeof(BlockReference))
                            {
                                BlockReference aBlockReference = (BlockReference)aEntity;  
                                if (aBlockReference.Name != "Marker")
                                {
                                    aObjectId = aBlockReference.ObjectId;
                                }
                            }
                            if (aEntity.GetType() == typeof(Solid3d))
                            {
                                Solid3d aSolid3d = (Solid3d)aEntity;
                                aObjectId = aSolid3d.ObjectId;
                            }
                        }
                    }
                }
            }
            return aObjectId;
        }
        public ObjectId AddBlockMarker()
        {
            //insert block marker
            ObjectId MarkerID = new ObjectId();
            try
            {
                    Point3d aPoint3d = selectpoint();
                    DocumentLock docLock = ProjectDocument.LockDocument();
                    MarkerID = ManipulateObject.CreateMarker(aPoint3d);
                    docLock.Dispose();
            }
            catch
            {
            }

            return MarkerID;
        }

        //get part marker 

        public Point3d GetPartCentroid(ObjectId aObjectId)
        {

            Point3d aPoint3d = new Point3d(0, 0, 0);
            Database db = ProjectDocument.Database;
            using (Transaction Trans = db.TransactionManager.StartTransaction())
            {
                try
                {
                    Entity aEntity = (Entity)Trans.GetObject(aObjectId, OpenMode.ForRead);
                    if (aEntity.GetType() == typeof(Solid3d))
                    {
                        Solid3d aSolid3d = (Solid3d)aEntity;
                        aPoint3d = aSolid3d.MassProperties.Centroid;
                    }
                    if (aEntity.GetType() == typeof(BlockReference))
                    {
                        BlockReference aBlockRef = (BlockReference)aEntity;
                        if (aBlockRef.BlockName != "Marker") aPoint3d = aBlockRef.Position;
                    }
                }
                catch
                {
                }
            }
            return aPoint3d;
        }
        public Point3d GetMarkerCentroid(ObjectId aObjectId)
        {

            Point3d aPoint3d = new Point3d(0, 0, 0);
            Database db = ProjectDocument.Database;
            using (Transaction Trans = db.TransactionManager.StartTransaction())
            {
                try
                {
                    Entity aEntity = (Entity)Trans.GetObject(aObjectId, OpenMode.ForRead);
                    if (aEntity.GetType() == typeof(BlockReference))
                    {
                        BlockReference aBlockRef = (BlockReference)aEntity;
                        if (aBlockRef.Name == "Marker") aPoint3d = aBlockRef.Position;
                    }
                }
                catch
                {
                }
            }
            return aPoint3d;
        }
        public Point3d GetMarkerDistance()
        {
            ACMPart aACMPart = SearchIncACMPart(SelectedACMMarker.PartName);
            Point3d aPoint3d = new Point3d(0, 0, 0);
            Point3d aPartCentroid = GetPartCentroid(aACMPart.ObjectID);
            Point3d aMarkerCentroid = GetMarkerCentroid(SelectedACMMarker.ObjectID);
            if ((aPartCentroid != null) && (aMarkerCentroid != null))
            {
                aPoint3d = new Point3d((aMarkerCentroid.X - aPartCentroid.X), (aMarkerCentroid.Y - aPartCentroid.Y), (aMarkerCentroid.Z - aPartCentroid.Z));
            }
            return aPoint3d;
        }

        //check part , marker
        public bool CheckMarkerExistance(string MarkerName)
        {
            return true;
        }


        ///////////////////////////////////////////
        /////////////////treenode//////////////////
        ///////////////////////////////////////////

        public void AddAssemblyMarkerNode(string MarkerName)
        {
            ACMTreeView.Nodes["Assembly"].Nodes["Markers"].Nodes.Add(MarkerName, MarkerName);
            ACMTreeView.Nodes["Assembly"].Nodes["Markers"].ExpandAll();        
        }
        public void AddPartNode(string PartName)
        {
            ACMTreeView.Nodes["Assembly"].Nodes["Parts"].Nodes.Add(PartName, PartName);
            ACMTreeView.Nodes["Assembly"].Nodes["Parts"].Expand();
            ACMTreeView.Nodes["Assembly"].Nodes["Parts"].Nodes[PartName].Nodes.Add("Markers", "Markers");
        }
        public void AddMarkerNode(string PartName, string MarkerName)
        {
            ACMTreeView.Nodes["Assembly"].Nodes["Parts"].Nodes[PartName].Nodes["Markers"].Nodes.Add(MarkerName, MarkerName);
            ACMTreeView.Nodes["Assembly"].Nodes["Parts"].Nodes[PartName].Nodes["Markers"].Expand();
        }
        public void AddJointNode(string JointName, string MarkerI, string MarkerJ)
        {
            ACMTreeView.Nodes["Assembly"].Nodes["Joints"].Nodes.Add(JointName, JointName);
            ACMTreeView.Nodes["Assembly"].Nodes["Joints"].Expand();
            ACMTreeView.Nodes["Assembly"].Nodes["Joints"].Nodes[JointName].Nodes.Add("Markers", "Markers");
            ACMTreeView.Nodes["Assembly"].Nodes["Joints"].Nodes[JointName].Nodes["Markers"].Nodes.Add(MarkerI, MarkerI);
            ACMTreeView.Nodes["Assembly"].Nodes["Joints"].Nodes[JointName].Nodes["Markers"].Nodes.Add(MarkerJ, MarkerJ);
        }

        ///////////////////////////////////////////
        //////////////////ACM//////////////////////

        public ACMMarker Get_ACMMarker(string aName)
        {
            foreach (ACMMarker aACMMarker in aACMAssembly.cACMMarker)
            {
                if (aACMMarker.Name == aName)
                {
                    return aACMMarker;
                }
            }
            foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
            {
                foreach (ACMMarker aACMMarker in aACMPart.cACMMarker)
                {
                    if (aACMMarker.Name == aName)
                    {
                        return aACMMarker;
                    }
                }
            }
            return null;
        }

        public void AddACMAssembly()
        {
            try
            {
                //new ACMAssembly
                aACMAssembly = new ACMAssembly();
                aACMAssembly.Name = AssemblyDocName;

                //event
                ACMMainClass.ACMEvent MotionEventObject = new ACMMainClass.ACMEvent(ProjectDocument.Database);

                //treenode
                ACMTreeView.Nodes.Clear();
                InitialiseTreeNode();
            }
            catch 
            { 
            }
        }
        public void AddAssemblyACMMarker(string MarkerName, ObjectId MarkerObjectId)
        {
            //new ACMMarker
            ACMMarker aACMMarker = new ACMMarker();
            aACMMarker.Name = MarkerName;
            aACMMarker.ObjectID = MarkerObjectId;

            //
            aACMAssembly.cACMMarker.Add(aACMMarker);

            //treenode
            AddAssemblyMarkerNode(MarkerName);
        }
        public void AddACMPart(string PartName, ObjectId PartObjectId)
        {
            try
            {
                //new ACMPart
                ACMPart aACMPart = new ACMPart();
                aACMPart.Name = PartName;
                aACMPart.ObjectID = PartObjectId;

                //
                aACMAssembly.cACMPart.Add(aACMPart);

                //treenode
                AddPartNode(PartName);
            }
            catch
            {
            }
        }
        public void AddACMMarker(string MarkerName, ObjectId MarkerObjectId)
        {
            try
            {
                //new ACMMarker
                ACMMarker aACMMarker = new ACMMarker();
                aACMMarker.Name = MarkerName;
                aACMMarker.PartName = SelectedACMPart.Name;
                aACMMarker.ObjectID = MarkerObjectId;

                //
                SelectedACMPart.cACMMarker.Add(aACMMarker);

                //treenode
                AddMarkerNode(SelectedACMPart.Name, MarkerName);
            }
            catch
            {
            }
        }
        public void AddACMJoint(string JointName, string MarkerI, string MarkerJ, string JointType)
        {
            try
            {
                //new ACMJoint
                ACMJoint aACMJoint = new ACMJoint();
                aACMJoint.Name = JointName;
                aACMJoint.MarkerI = MarkerI;
                aACMJoint.MarkerJ = MarkerJ;
                aACMJoint.JointType = JointType;

                //
                aACMAssembly.cACMJoint.Add(aACMJoint);

                //treenode
                AddJointNode(aACMJoint.Name, aACMJoint.MarkerI, aACMJoint.MarkerJ);
            }
            catch
            {
            }
        }
        public void AddACMMotion()
        {

        }
        public void AddACMSimulation(double Tstart, double Tend, double Hout, double Hmin, double Hmax, double ErrorTolerance)
        {
            try
            {
                // new ACMSimulation
                ACMSimulation aACMSimulation = new ACMSimulation();
                aACMSimulation.Tstart = Tstart;
                aACMSimulation.Tend = Tend;
                aACMSimulation.Hout = Hout;
                aACMSimulation.Hmin = Hmin;
                aACMSimulation.Hmax = Hmax;
                aACMSimulation.ErrorTolerance = ErrorTolerance;
                //

                aACMAssembly.aACMSimulation = aACMSimulation;
                
            }
            catch
            {
            }
        }

        ////////////////////////////////////////////////////////////////////////////// 
        //////////////////////////////////// CADSM ///////////////////////////////////  
        //////////////////////////////////////////////////////////////////////////////  
 
        public void AddCSMAssembly()
        {
            try
            {
                //ACMAssembly
                aACMAssembly.aCSMAssembly = (CSMAssembly)COMCADSM.AddAssembly();
                aACMAssembly.aCSMAssembly.Name = aACMAssembly.Name;

                double[] aGravity = { 10.0, 0, 0 };
                aACMAssembly.aCSMAssembly.Gravity = aGravity;

                //ACMMarker
                ACMMarker aACMMarker = new ACMMarker();
                aACMMarker.aCSMMarker = (CSMMarker)aACMAssembly.aCSMAssembly.AddMarker();
                aACMMarker.Name = "AssemblyMarker";
                aACMMarker.aCSMMarker.Name = "AssemblyMarker";
                double[] aAOriginMarkerMFf = { 1.0, 0.0, 0.0, 0.0, 
                                               0.0, 1.0, 0.0, 0.0, 
                                               0.0, 0.0, 1.0, 0.0, 
                                               0.0, 0.0, 0.0, 1.0 };

                aACMMarker.aCSMMarker.MFf = aAOriginMarkerMFf;

                aACMAssembly.aASMCSMMarker = aACMMarker;
            }
            catch
            {
            }
        }
        public void AddCSMAssemblyMarker(ACMMarker aACMMarker)
        {
            try
            {
                aACMMarker.aCSMMarker = (CSMMarker)aACMAssembly.aCSMAssembly.AddMarker();
                aACMMarker.aCSMMarker.Name = aACMMarker.Name;

                Point3d aMarkerCentroid = GetMarkerCentroid(aACMMarker.ObjectID);

                double[] aMarkerMFf = { 1.0, 0.0, 0.0, aMarkerCentroid.X, 
                                        0.0, 1.0, 0.0, aMarkerCentroid.Y, 
                                        0.0, 0.0, 1.0, aMarkerCentroid.Z, 
                                        0.0, 0.0, 0.0, 1.0 };

                aACMMarker.aCSMMarker.MFf = aMarkerMFf;
            }
            catch
            {
            }

        }
        public void AddCSMPart(ACMPart aACMPart)
        {
            try
            {
                //CSMPart
                aACMPart.aCSMPart = (CSMPart)aACMAssembly.aCSMAssembly.AddPart();
                aACMPart.aCSMPart.Name = aACMPart.Name;
                
                Point3d aPoint3d = GetPartCentroid(aACMPart.ObjectID);

                double[] aMFf = { 1.0, 0.0, 0.0, aPoint3d.X, 
                                  0.0, 1.0, 0.0, aPoint3d.Y, 
                                  0.0, 0.0, 1.0, aPoint3d.Z, 
                                  0.0, 0.0, 0.0, 1.0 };

                aACMPart.aCSMPart.MFf = aMFf;

                double[] avOfO = { 0, 0, 0 };
                aACMPart.aCSMPart.vOfO = avOfO;
                double[] aOmeOfO = { 0, 0, 0 };
                aACMPart.aCSMPart.OmeOfO = aOmeOfO;

                //CSMMassMarker

                aACMPart.aCSMMassMarker = (CSMMassMarker)aACMPart.aCSMPart.AddMassMarker();
                
                double aMass = 5.0;
                aACMPart.aCSMMassMarker.Mass = aMass;
                
                double[] aIxxIyyIzzIxyIxzIyzCM = { 1, 1, 1, 0, 0, 0 };
                aACMPart.aCSMMassMarker.IxxIyyIzzIxyIxzIyzCM = aIxxIyyIzzIxyIxzIyzCM;
            }
            catch
            {
            }
        }
        public void AddCSMMarker(ACMMarker aACMMarker)
        {
            try
            {
                //CSMMarker
                ACMPart aACMPart = SearchIncACMPart(aACMMarker.PartName);

                aACMMarker.aCSMMarker = (CSMMarker)aACMPart.aCSMPart.AddMarker();
                aACMMarker.aCSMMarker.Name = aACMMarker.Name;

                Point3d aMarkerCentroid = GetMarkerCentroid(aACMMarker.ObjectID);
                Point3d aPartCentroid = GetPartCentroid(aACMPart.ObjectID);

                Point3d aPoint3d = new Point3d((aMarkerCentroid.X - aPartCentroid.X), (aMarkerCentroid.Y - aPartCentroid.Y), (aMarkerCentroid.Z - aPartCentroid.Z));

                double[] aMarkerMFf = { 1.0, 0.0, 0.0, aPoint3d.X, 
                                        0.0, 1.0, 0.0, aPoint3d.Y, 
                                        0.0, 0.0, 1.0, aPoint3d.Z, 
                                        0.0, 0.0, 0.0, 1.0 };

                aACMMarker.aCSMMarker.MFf = aMarkerMFf;
            }
            catch
            {
            }
        }
        public void AddCSMRevoluteJoint(string JointName, ACMMarker markerI, ACMMarker markerJ)
        {
            try
            {
                ACMJoint aACMJoint = new ACMJoint();
                aACMJoint.Name = JointName;
                aACMJoint.MarkerI = markerI.Name;
                aACMJoint.MarkerJ = markerJ.Name;
                aACMJoint.aCSMRevoluteJoint = (CSMRevoluteJoint)aACMAssembly.aCSMAssembly.AddRevoluteJoint();
                aACMJoint.aCSMRevoluteJoint.MarkerI = markerI.aCSMMarker.FullName();
                aACMJoint.aCSMRevoluteJoint.MarkerJ = markerJ.aCSMMarker.FullName();
                aACMAssembly.cACMJoint.Add(aACMJoint);

                AddJointNode(aACMJoint.Name, aACMJoint.MarkerI, aACMJoint.MarkerJ);
            }
            catch
            {
            }
        }
        public void AddCSMRotationalMotion(ACMJoint aACMJoint, string aFormula)
        {
            ACMMotion aACMMotion = new ACMMotion();
            aACMMotion.aCSMRotationalMotion = (CSMRotationalMotion)aACMJoint.aCSMRevoluteJoint.AddRotation();
            aACMMotion.aCSMRotationalMotion.Rotation = aFormula;

            aACMAssembly.cACMMotion.Add(aACMMotion);
        }
        public void AddCSMSimulationParameters()
        {
            try
            {

                ACMSimulation aaACMSimu = new ACMSimulation();

                aACMAssembly.aACMSimulation = aaACMSimu;
                //CSM
                double Tstart = 0.0, Tend = 1.0, Hout = 0.01, Hmin = 1.0E-9, Hmax = 1.0, ErrorTolerance = 1.0E-6;

                aACMAssembly.aACMSimulation.aCSMSimulation = (CSMSimulation)aACMAssembly.aCSMAssembly.SimulationParameters();
                aACMAssembly.aACMSimulation.aCSMSimulation.Tstart = Tstart;
                aACMAssembly.aACMSimulation.aCSMSimulation.Tend = Tend;
                aACMAssembly.aACMSimulation.aCSMSimulation.Hout = Hout;
                aACMAssembly.aACMSimulation.aCSMSimulation.Hmin = Hmin;
                aACMAssembly.aACMSimulation.aCSMSimulation.Hmax = Hmax;
                aACMAssembly.aACMSimulation.aCSMSimulation.ErrorTolerance = ErrorTolerance;

            }
            catch
            {
            }
        }
        public void StartDynamic()
        {
            try
            {
                COMCADSM.StartDynamic();
            }
            catch 
            { 
            }
        }
        public void StartKinematic()
        {
            try
            {
                COMCADSM.StartKinematic();
            }
            catch
            {
            }
        }

        public void StartSimulate()
        {
            try
            {
                SimulationStatus = false;
                AddCSMAssembly();
                foreach (ACMMarker aACMMarker in aACMAssembly.cACMMarker)
                {
                    AddCSMAssemblyMarker(aACMMarker);
                }
                foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                {
                    AddCSMPart(aACMPart);
                    foreach (ACMMarker aACMMarker in aACMPart.cACMMarker)
                    {
                        AddCSMMarker(aACMMarker);
                    }
                }
                foreach (ACMJoint aACMJoint in aACMAssembly.cACMJoint)
                {
                    if (aACMJoint.JointType == "Fixed")
                    {
                        CSMFixedJoint aCSMFixedJoint = (CSMFixedJoint)aACMAssembly.aCSMAssembly.AddFixedJoint();
                        aCSMFixedJoint.MarkerI = aACMJoint.MarkerI;
                        aCSMFixedJoint.MarkerJ = aACMJoint.MarkerJ;
                    }
                    if (aACMJoint.JointType == "RevoluteZ")
                    {
                        aACMJoint.aCSMRevoluteJoint = (CSMRevoluteJoint)aACMAssembly.aCSMAssembly.AddRevoluteJoint();
                        ACMMarker ACMMarkerI = Get_ACMMarker(aACMJoint.MarkerI);
                        ACMMarker ACMMarkerJ = Get_ACMMarker(aACMJoint.MarkerJ);
                        aACMJoint.aCSMRevoluteJoint.MarkerI = ACMMarkerI.aCSMMarker.FullName();
                        aACMJoint.aCSMRevoluteJoint.MarkerJ = ACMMarkerJ.aCSMMarker.FullName();

                        AddCSMRotationalMotion(aACMJoint, "2*pi*time");
                    }
                }

                AddCSMSimulationParameters();
                SimulationStatus = true;
            }
            catch 
            { 
            }
        }


        //
        private void ACMTreeView_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            try
            {
                if (e.Button == MouseButtons.Right)
                {
                }
            }
            catch
            {
            }
        }

        public Boolean testanimation()
        {
            try
            {
                short NoOfFrames = (short)COMCADSM.NumberOfFrames();
                TotalNoOfFrame = (short)COMCADSM.NumberOfFrames();
                short i = 1;
                while (i <= NoOfFrames)
                {
                    bool check_frame = COMCADSM.CheckFrame(i);
                    double aTimeAt = (double)aACMAssembly.aCSMAssembly.TimeAt(i);
                    double[] aPosVelAccData = (double[])aACMAssembly.aCSMAssembly.PosVelAccData(i);
                    foreach (ACMMarker aACMMarker in aACMAssembly.cACMMarker)
                    {
                        double[] aPos4by4VelAccData = (double[])aACMMarker.aCSMMarker.Pos4by4VelAccData(i);
                    }
                    foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                    {
                        aACMPart.cPosRotMatVelAccData.Add((double[])aACMPart.aCSMPart.PosRotMatVelAccData(i));
                        foreach (ACMMarker aACMMarker in aACMPart.cACMMarker)
                        {
                            double[] aMarker_PosRotMatVelAccData = (double[])aACMMarker.aCSMMarker.PosRotMatVelAccData(i);
                        }
                    }
                    foreach (ACMJoint aACMJoint in aACMAssembly.cACMJoint)
                    {
                        Array aArray = (Array)aACMJoint.aCSMRevoluteJoint.ForceTorqueData(i);
                    }
                    i++;
                }
                return true;
            }
            catch
            {
                return false;
            }
        }
        [CommandMethod("et")]
        public void enabletime()
        {
            aTimer.Enabled = true;
        }

        [CommandMethod("dt")]
        public void disabletime()
        {
            aTimer.Enabled = false;
        }



        [CommandMethod("testa")]
        public void testanimate()
        {
            try
            {
                using (DocumentLock aDocumentLock = ProjectDocument.LockDocument())
                {
                    short NoOfFrames = (short)COMCADSM.NumberOfFrames();

                    for (short i = 1; i <= NoOfFrames; i++)
                    {
                        foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                        {
                            if (i == 1)
                            {
                                double[] aPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[i - 1];
                                double Xposition = aPosRotMatVelAccData[0];
                                double Yposition = aPosRotMatVelAccData[1];
                                double Zposition = aPosRotMatVelAccData[2];
                                double Matrix1 = aPosRotMatVelAccData[3];
                                double Matrix2 = aPosRotMatVelAccData[4];
                                double Matrix3 = aPosRotMatVelAccData[5];
                                double Matrix4 = aPosRotMatVelAccData[6];
                                double Matrix5 = aPosRotMatVelAccData[7];
                                double Matrix6 = aPosRotMatVelAccData[8];
                                double Matrix7 = aPosRotMatVelAccData[9];
                                double Matrix8 = aPosRotMatVelAccData[10];
                                double Matrix9 = aPosRotMatVelAccData[11];
                                using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
                                {
                                    double[] adouble = { Matrix1, Matrix2, Matrix3, Xposition, Matrix4, Matrix5, Matrix6, Yposition, Matrix7, Matrix8, Matrix9, Zposition, 0.0, 0.0, 0.0, 1.0 };
                                    Matrix3d aMatrix3d = new Matrix3d(adouble);
                                    Entity aEntity = (Entity)Trans.GetObject(aACMPart.ObjectID, OpenMode.ForRead);
                                    Point3d aPoint3d = aEntity.Database.WorldUcsBaseOrigin(0);
                                    aEntity.UpgradeOpen();
                                    aEntity.TransformBy(aMatrix3d);
                                    Trans.Commit();
                                }
                            }
                            else
                                if (i > 1)
                                {
                                    double[] aPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[i - 2];
                                    double[] bPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[i - 1];
                                    double aXposition = aPosRotMatVelAccData[0];
                                    double aYposition = aPosRotMatVelAccData[1];
                                    double aZposition = aPosRotMatVelAccData[2];
                                    double aMatrix1 = aPosRotMatVelAccData[3];
                                    double aMatrix2 = aPosRotMatVelAccData[4];
                                    double aMatrix3 = aPosRotMatVelAccData[5];
                                    double aMatrix4 = aPosRotMatVelAccData[6];
                                    double aMatrix5 = aPosRotMatVelAccData[7];
                                    double aMatrix6 = aPosRotMatVelAccData[8];
                                    double aMatrix7 = aPosRotMatVelAccData[9];
                                    double aMatrix8 = aPosRotMatVelAccData[10];
                                    double aMatrix9 = aPosRotMatVelAccData[11];
                                    //
                                    double bXposition = bPosRotMatVelAccData[0];
                                    double bYposition = bPosRotMatVelAccData[1];
                                    double bZposition = bPosRotMatVelAccData[2];
                                    double bMatrix1 = bPosRotMatVelAccData[3];
                                    double bMatrix2 = bPosRotMatVelAccData[4];
                                    double bMatrix3 = bPosRotMatVelAccData[5];
                                    double bMatrix4 = bPosRotMatVelAccData[6];
                                    double bMatrix5 = bPosRotMatVelAccData[7];
                                    double bMatrix6 = bPosRotMatVelAccData[8];
                                    double bMatrix7 = bPosRotMatVelAccData[9];
                                    double bMatrix8 = bPosRotMatVelAccData[10];
                                    double bMatrix9 = bPosRotMatVelAccData[11];
                                    using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
                                    {
                                        double[] adouble = { aMatrix1, aMatrix2, aMatrix3, aXposition, aMatrix4, aMatrix5, aMatrix6, aYposition, aMatrix7, aMatrix8, aMatrix9, aZposition, 0.0, 0.0, 0.0, 1.0 };
                                        double[] bdouble = { bMatrix1, bMatrix2, bMatrix3, bXposition, bMatrix4, bMatrix5, bMatrix6, bYposition, bMatrix7, bMatrix8, bMatrix9, bZposition, 0.0, 0.0, 0.0, 1.0 };
                                        Matrix3d aMatrix3d = new Matrix3d(adouble);
                                        Matrix3d inverseA = aMatrix3d.Inverse();
                                        Matrix3d bMatrix3d = new Matrix3d(bdouble);
                                        Matrix3d cMatrix = bMatrix3d.PreMultiplyBy(inverseA);
                                        Entity aEntity = (Entity)Trans.GetObject(aACMPart.ObjectID, OpenMode.ForRead);
                                        Point3d aPoint3d = aEntity.Database.WorldUcsBaseOrigin(0);
                                        aEntity.UpgradeOpen();
                                        aEntity.TransformBy(cMatrix);
                                        Trans.Commit();
                                    }
                                }
                        }
                    }
                    foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                    {
                        double[] ddPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[0];
                        double dXposition = ddPosRotMatVelAccData[0];
                        double dYposition = ddPosRotMatVelAccData[1];
                        double dZposition = ddPosRotMatVelAccData[2];
                        double dMatrix1 = ddPosRotMatVelAccData[3];
                        double dMatrix2 = ddPosRotMatVelAccData[4];
                        double dMatrix3 = ddPosRotMatVelAccData[5];
                        double dMatrix4 = ddPosRotMatVelAccData[6];
                        double dMatrix5 = ddPosRotMatVelAccData[7];
                        double dMatrix6 = ddPosRotMatVelAccData[8];
                        double dMatrix7 = ddPosRotMatVelAccData[9];
                        double dMatrix8 = ddPosRotMatVelAccData[10];
                        double dMatrix9 = ddPosRotMatVelAccData[11];
                        using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
                        {
                            double[] ddouble = { dMatrix1, dMatrix2, dMatrix3, dXposition, dMatrix4, dMatrix5, dMatrix6, dYposition, dMatrix7, dMatrix8, dMatrix9, dZposition, 0.0, 0.0, 0.0, 1.0 };
                            Matrix3d aMatrix3d = new Matrix3d(ddouble);
                            Matrix3d inverseA = aMatrix3d.Inverse();
                            Entity aEntity = (Entity)Trans.GetObject(aACMPart.ObjectID, OpenMode.ForRead);
                            Point3d aPoint3d = aEntity.Database.WorldUcsBaseOrigin(0);
                            aEntity.UpgradeOpen();
                            aEntity.TransformBy(inverseA);
                            Trans.Commit();
                        }
                    }
                }
            }
            catch
            {
            }

        }


        public double[] inverseMatrix(double[] aDouble)
        {
            double a = aDouble[0];
            double b = aDouble[1];
            double c = aDouble[2];
            double d = aDouble[3];
            double e = aDouble[4];
            double f = aDouble[5];
            double g = aDouble[6];
            double h = aDouble[7];
            double i = aDouble[8];

            double z = a * ((e * i) - (f * h)) + b*((f * g) - (i * d)) + c*((d * h) - (e * g));

            aDouble[0] = a / z;
            aDouble[1] = d / z;
            aDouble[2] = g / z;
            aDouble[3] = b / z;
            aDouble[4] = e / z;
            aDouble[5] = h / z;
            aDouble[6] = c / z;
            aDouble[7] = f / z;
            aDouble[8] = i / z;

            return aDouble;
        }
        //private static void On
        [CommandMethod("f1")]
        public void aMovePartByFrame()
        {
            try
            {
                short aFrame = 1;
                using (DocumentLock aDocumentLock = ProjectDocument.LockDocument())
                {
                    foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                    {
                        double[] aPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[aFrame - 1];
                        double Xposition = aPosRotMatVelAccData[0];
                        double Yposition = aPosRotMatVelAccData[1];
                        double Zposition = aPosRotMatVelAccData[2];
                        double Matrix1 = aPosRotMatVelAccData[3];
                        double Matrix2 = aPosRotMatVelAccData[4];
                        double Matrix3 = aPosRotMatVelAccData[5];
                        double Matrix4 = aPosRotMatVelAccData[6];
                        double Matrix5 = aPosRotMatVelAccData[7];
                        double Matrix6 = aPosRotMatVelAccData[8];
                        double Matrix7 = aPosRotMatVelAccData[9];
                        double Matrix8 = aPosRotMatVelAccData[10];
                        double Matrix9 = aPosRotMatVelAccData[11];
                        using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
                        {
                            double[] adouble = { Matrix1, Matrix2, Matrix3, Xposition, Matrix4, Matrix5, Matrix6, Yposition, Matrix7, Matrix8, Matrix9, Zposition, 0.0, 0.0, 0.0, 1.0 };
                            Matrix3d aMatrix3d = new Matrix3d(adouble);
                            Entity aEntity = (Entity)Trans.GetObject(aACMPart.ObjectID, OpenMode.ForRead);
                            Point3d aPoint3d = aEntity.Database.WorldUcsBaseOrigin(0);
                            aEntity.UpgradeOpen();
                            aEntity.TransformBy(aMatrix3d);
                            Trans.Commit();
                        }
                    }
                }
            }
            catch
            {
            }
        }

        [CommandMethod("f2")]
        public void bMovePartByFrame()
        {
            try
            {
                short aFrame = 2;
                using (DocumentLock aDocumentLock = ProjectDocument.LockDocument())
                {
                    foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                    {
                        double[] aPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[aFrame - 1];
                        double Xposition = aPosRotMatVelAccData[0];
                        double Yposition = aPosRotMatVelAccData[1];
                        double Zposition = aPosRotMatVelAccData[2];
                        double Matrix1 = aPosRotMatVelAccData[3];
                        double Matrix2 = aPosRotMatVelAccData[4];
                        double Matrix3 = aPosRotMatVelAccData[5];
                        double Matrix4 = aPosRotMatVelAccData[6];
                        double Matrix5 = aPosRotMatVelAccData[7];
                        double Matrix6 = aPosRotMatVelAccData[8];
                        double Matrix7 = aPosRotMatVelAccData[9];
                        double Matrix8 = aPosRotMatVelAccData[10];
                        double Matrix9 = aPosRotMatVelAccData[11];
                        using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
                        {
                            double[] adouble = { Matrix1, Matrix2, Matrix3, Xposition, Matrix4, Matrix5, Matrix6, Yposition, Matrix7, Matrix8, Matrix9, Zposition, 0.0, 0.0, 0.0, 1.0 };
                            Matrix3d aMatrix3d = new Matrix3d(adouble);
                            Entity aEntity = (Entity)Trans.GetObject(aACMPart.ObjectID, OpenMode.ForRead);
                            Point3d aPoint3d = aEntity.Database.WorldUcsBaseOrigin(0);
                            aEntity.UpgradeOpen();
                            aEntity.TransformBy(aMatrix3d);
                            Trans.Commit();
                        }
                    }
                }
            }
            catch
            {
            }
        }

        [CommandMethod("f3")]
        public void cMovePartByFrame()
        {
            try
            {
                short aFrame = 3;
                using (DocumentLock aDocumentLock = ProjectDocument.LockDocument())
                {
                    foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                    {
                        double[] aPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[aFrame - 1];
                        double Xposition = aPosRotMatVelAccData[0];
                        double Yposition = aPosRotMatVelAccData[1];
                        double Zposition = aPosRotMatVelAccData[2];
                        double Matrix1 = aPosRotMatVelAccData[3];
                        double Matrix2 = aPosRotMatVelAccData[4];
                        double Matrix3 = aPosRotMatVelAccData[5];
                        double Matrix4 = aPosRotMatVelAccData[6];
                        double Matrix5 = aPosRotMatVelAccData[7];
                        double Matrix6 = aPosRotMatVelAccData[8];
                        double Matrix7 = aPosRotMatVelAccData[9];
                        double Matrix8 = aPosRotMatVelAccData[10];
                        double Matrix9 = aPosRotMatVelAccData[11];
                        using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
                        {
                            double[] adouble = { Matrix1, Matrix2, Matrix3, Xposition, Matrix4, Matrix5, Matrix6, Yposition, Matrix7, Matrix8, Matrix9, Zposition, 0.0, 0.0, 0.0, 1.0 };
                            Matrix3d aMatrix3d = new Matrix3d(adouble);
                            aMatrix3d.Inverse();
                            
                            Entity aEntity = (Entity)Trans.GetObject(aACMPart.ObjectID, OpenMode.ForRead);
                            Point3d aPoint3d = aEntity.Database.WorldUcsBaseOrigin(0);
                            aEntity.UpgradeOpen();
                            aEntity.TransformBy(aMatrix3d);
                            Trans.Commit();
                        }
                    }
                }
            }
            catch
            {
            }
        }

        [CommandMethod("f4")]
        public void dMovePartByFrame()
        {
            try
            {
                short aFrame = 4;
                using (DocumentLock aDocumentLock = ProjectDocument.LockDocument())
                {
                    foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                    {
                        double[] aPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[aFrame - 1];
                        double Xposition = aPosRotMatVelAccData[0];
                        double Yposition = aPosRotMatVelAccData[1];
                        double Zposition = aPosRotMatVelAccData[2];
                        double Matrix1 = aPosRotMatVelAccData[3];
                        double Matrix2 = aPosRotMatVelAccData[4];
                        double Matrix3 = aPosRotMatVelAccData[5];
                        double Matrix4 = aPosRotMatVelAccData[6];
                        double Matrix5 = aPosRotMatVelAccData[7];
                        double Matrix6 = aPosRotMatVelAccData[8];
                        double Matrix7 = aPosRotMatVelAccData[9];
                        double Matrix8 = aPosRotMatVelAccData[10];
                        double Matrix9 = aPosRotMatVelAccData[11];
                        using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
                        {
                            double[] adouble = { Matrix1, Matrix2, Matrix3, Xposition, Matrix4, Matrix5, Matrix6, Yposition, Matrix7, Matrix8, Matrix9, Zposition, 0.0, 0.0, 0.0, 1.0 };
                            Matrix3d aMatrix3d = new Matrix3d(adouble);
                            Entity aEntity = (Entity)Trans.GetObject(aACMPart.ObjectID, OpenMode.ForRead);
                            Point3d aPoint3d = aEntity.Database.WorldUcsBaseOrigin(0);
                            aEntity.UpgradeOpen();
                            aEntity.TransformBy(aMatrix3d);
                            Trans.Commit();
                        }
                    }
                }
            }
            catch
            {
            }
        }

        [CommandMethod("f5")]
        public void eMovePartByFrame()
        {
            try
            {
                short aFrame = 5;
                using (DocumentLock aDocumentLock = ProjectDocument.LockDocument())
                {
                    foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                    {
                        double[] aPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[aFrame - 1];
                        double Xposition = aPosRotMatVelAccData[0];
                        double Yposition = aPosRotMatVelAccData[1];
                        double Zposition = aPosRotMatVelAccData[2];
                        double Matrix1 = aPosRotMatVelAccData[3];
                        double Matrix2 = aPosRotMatVelAccData[4];
                        double Matrix3 = aPosRotMatVelAccData[5];
                        double Matrix4 = aPosRotMatVelAccData[6];
                        double Matrix5 = aPosRotMatVelAccData[7];
                        double Matrix6 = aPosRotMatVelAccData[8];
                        double Matrix7 = aPosRotMatVelAccData[9];
                        double Matrix8 = aPosRotMatVelAccData[10];
                        double Matrix9 = aPosRotMatVelAccData[11];
                        using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
                        {
                            double[] adouble = { Matrix1, Matrix2, Matrix3, Xposition, Matrix4, Matrix5, Matrix6, Yposition, Matrix7, Matrix8, Matrix9, Zposition, 0.0, 0.0, 0.0, 1.0 };
                            Matrix3d aMatrix3d = new Matrix3d(adouble);
                            Entity aEntity = (Entity)Trans.GetObject(aACMPart.ObjectID, OpenMode.ForRead);
                            Point3d aPoint3d = aEntity.Database.WorldUcsBaseOrigin(0);
                            aEntity.UpgradeOpen();
                            aEntity.TransformBy(aMatrix3d);
                            Trans.Commit();
                        }
                    }
                }
            }
            catch
            {
            }
        }

        [CommandMethod("f6")]
        public void fMovePartByFrame()
        {
            try
            {
                short aFrame = 6;
                using (DocumentLock aDocumentLock = ProjectDocument.LockDocument())
                {
                    foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                    {
                        double[] aPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[aFrame - 1];
                        double Xposition = aPosRotMatVelAccData[0];
                        double Yposition = aPosRotMatVelAccData[1];
                        double Zposition = aPosRotMatVelAccData[2];
                        double Matrix1 = aPosRotMatVelAccData[3];
                        double Matrix2 = aPosRotMatVelAccData[4];
                        double Matrix3 = aPosRotMatVelAccData[5];
                        double Matrix4 = aPosRotMatVelAccData[6];
                        double Matrix5 = aPosRotMatVelAccData[7];
                        double Matrix6 = aPosRotMatVelAccData[8];
                        double Matrix7 = aPosRotMatVelAccData[9];
                        double Matrix8 = aPosRotMatVelAccData[10];
                        double Matrix9 = aPosRotMatVelAccData[11];
                        using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
                        {
                            double[] adouble = { Matrix1, Matrix2, Matrix3, Xposition, Matrix4, Matrix5, Matrix6, Yposition, Matrix7, Matrix8, Matrix9, Zposition, 0.0, 0.0, 0.0, 1.0 };
                            Matrix3d aMatrix3d = new Matrix3d(adouble);
                            Entity aEntity = (Entity)Trans.GetObject(aACMPart.ObjectID, OpenMode.ForRead);
                            Point3d aPoint3d = aEntity.Database.WorldUcsBaseOrigin(0);
                            aEntity.UpgradeOpen();
                            aEntity.TransformBy(aMatrix3d);
                            Trans.Commit();
                        }
                    }
                }
            }
            catch
            {
            }
        }

        [CommandMethod("f7")]
        public void gMovePartByFrame()
        {
            try
            {
                short aFrame = 7;
                using (DocumentLock aDocumentLock = ProjectDocument.LockDocument())
                {
                    foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                    {
                        double[] aPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[aFrame - 1];
                        double Xposition = aPosRotMatVelAccData[0];
                        double Yposition = aPosRotMatVelAccData[1];
                        double Zposition = aPosRotMatVelAccData[2];
                        double Matrix1 = aPosRotMatVelAccData[3];
                        double Matrix2 = aPosRotMatVelAccData[4];
                        double Matrix3 = aPosRotMatVelAccData[5];
                        double Matrix4 = aPosRotMatVelAccData[6];
                        double Matrix5 = aPosRotMatVelAccData[7];
                        double Matrix6 = aPosRotMatVelAccData[8];
                        double Matrix7 = aPosRotMatVelAccData[9];
                        double Matrix8 = aPosRotMatVelAccData[10];
                        double Matrix9 = aPosRotMatVelAccData[11];
                        using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
                        {
                            double[] adouble = { Matrix1, Matrix2, Matrix3, Xposition, Matrix4, Matrix5, Matrix6, Yposition, Matrix7, Matrix8, Matrix9, Zposition, 0.0, 0.0, 0.0, 1.0 };
                            Matrix3d aMatrix3d = new Matrix3d(adouble);
                            Entity aEntity = (Entity)Trans.GetObject(aACMPart.ObjectID, OpenMode.ForRead);
                            Point3d aPoint3d = aEntity.Database.WorldUcsBaseOrigin(0);
                            aEntity.UpgradeOpen();
                            aEntity.TransformBy(aMatrix3d);
                            Trans.Commit();
                        }
                    }
                }
            }
            catch
            {
            }
        }

        public void MovePartByFrame(short aFrame)
        {
            try
            {
                using (DocumentLock aDocumentLock = ProjectDocument.LockDocument())
                {
                    foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                    {
                        double[] aPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[aFrame - 1];
                        double Xposition = aPosRotMatVelAccData[0];
                        double Yposition = aPosRotMatVelAccData[1];
                        double Zposition = aPosRotMatVelAccData[2];
                        double Matrix1 = aPosRotMatVelAccData[3];
                        double Matrix2 = aPosRotMatVelAccData[4];
                        double Matrix3 = aPosRotMatVelAccData[5];
                        double Matrix4 = aPosRotMatVelAccData[6];
                        double Matrix5 = aPosRotMatVelAccData[7];
                        double Matrix6 = aPosRotMatVelAccData[8];
                        double Matrix7 = aPosRotMatVelAccData[9];
                        double Matrix8 = aPosRotMatVelAccData[10];
                        double Matrix9 = aPosRotMatVelAccData[11];
                        using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
                        {
                            double[] adouble = { Matrix1, Matrix2, Matrix3, Xposition, Matrix4, Matrix5, Matrix6, Yposition, Matrix7, Matrix8, Matrix9, Zposition, 0.0, 0.0, 0.0, 1.0 };
                            Matrix3d aMatrix3d = new Matrix3d(adouble);
                            Entity aEntity = (Entity)Trans.GetObject(aACMPart.ObjectID, OpenMode.ForRead);
                            Point3d aPoint3d = aEntity.Database.WorldUcsBaseOrigin(0);
                            aEntity.UpgradeOpen();
                            aEntity.TransformBy(aMatrix3d);
                            Trans.Commit();
                        }
                    }
                }
            }
            catch
            {
            }
        }


        private static void movepart(Object source, ElapsedEventArgs e)
        {
            try 
            {
                using (DocumentLock aDocumentLock = ProjectDocument.LockDocument())
                {
                    foreach (ACMPart aACMPart in aACMAssembly.cACMPart)
                    {
                        double[] aPosRotMatVelAccData = (double[])aACMPart.cPosRotMatVelAccData[Current_Frame - 1];
                        double Xposition = aPosRotMatVelAccData[0];
                        double Yposition = aPosRotMatVelAccData[1];
                        double Zposition = aPosRotMatVelAccData[2];
                        double Matrix1 = aPosRotMatVelAccData[3];
                        double Matrix2 = aPosRotMatVelAccData[4];
                        double Matrix3 = aPosRotMatVelAccData[5];
                        double Matrix4 = aPosRotMatVelAccData[6];
                        double Matrix5 = aPosRotMatVelAccData[7];
                        double Matrix6 = aPosRotMatVelAccData[8];
                        double Matrix7 = aPosRotMatVelAccData[9];
                        double Matrix8 = aPosRotMatVelAccData[10];
                        double Matrix9 = aPosRotMatVelAccData[11];
                        using (Transaction Trans = ProjectDocument.TransactionManager.StartTransaction())
                        {
                            double[] adouble = { Matrix1, Matrix2, Matrix3, Xposition, Matrix4, Matrix5, Matrix6, Yposition, Matrix7, Matrix8, Matrix9, Zposition, 0.0, 0.0, 0.0, 1.0 };
                            Matrix3d aMatrix3d = new Matrix3d(adouble);
                            Entity aEntity = (Entity)Trans.GetObject(aACMPart.ObjectID, OpenMode.ForRead);
                            Point3d aPoint3d = aEntity.Database.WorldUcsBaseOrigin(0);
                            aEntity.UpgradeOpen();
                            aEntity.TransformBy(aMatrix3d);
                            Trans.Commit();
                        }
                    }
                }
                Current_Frame++;
                if (Current_Frame > TotalNoOfFrame)
                {
                    Current_Frame = 1;
                }
            }
                catch
            {
            }
        }






        //Joint
        private void MI_Fixed_Click(object sender, EventArgs e)
        {

        }

        public void testsimulation()
        {
            try
            {
                //assem
                CSMAssembly aCSMAssembly = (CSMAssembly)COMCADSM.AddAssembly();
                aCSMAssembly.Name = "test";

                double[] gravity = {10.0,0,0};

                aCSMAssembly.Gravity = gravity;

                CSMMarker aCSMMarker = (CSMMarker)aCSMAssembly.AddMarker();
                aCSMMarker.Name = "AssemblyMarker";
                double[] aAOriginMarkerMFf = { 1.0, 0.0, 0.0, 0.0, 
                                               0.0, 1.0, 0.0, 0.0, 
                                               0.0, 0.0, 1.0, 0.0, 
                                               0.0, 0.0, 0.0, 1.0 };
                aCSMMarker.MFf = aAOriginMarkerMFf;

                //assem marker
                CSMMarker bCSMMarker = (CSMMarker)aCSMAssembly.AddMarker();
                bCSMMarker.Name = "Mark1";
                double[] bAOriginMarkerMFf = { 1.0, 0.0, 0.0, 0.0, 
                                               0.0, 1.0, 0.0, 0.0, 
                                               0.0, 0.0, 1.0, 0.0, 
                                               0.0, 0.0, 0.0, 1.0 };
                bCSMMarker.MFf = bAOriginMarkerMFf;

                // part
                CSMPart aCSMPart = (CSMPart)aCSMAssembly.AddPart();
                aCSMPart.Name = "Part1";
                double[] cAOriginMarkerMFf = { 1.0, 0.0, 0.0, 0.0, 
                               0.0, 1.0, 0.0, 0.0, 
                               0.0, 0.0, 1.0, 0.0, 
                               0.0, 0.0, 0.0, 1.0 };

                aCSMPart.MFf = cAOriginMarkerMFf;
                double[] avOfO = { 0,0,0 };
                aCSMPart.vOfO = avOfO;
                double[] aOmeOfO = { 0,0,0 };
                aCSMPart.OmeOfO = aOmeOfO;
                CSMMassMarker aCSMMassMarker = (CSMMassMarker)aCSMPart.AddMassMarker();
                double mass = 5.0;
                aCSMMassMarker.Mass = mass;
                double[] aIxxIyyIzzIxyIxzIyzCM = { 1,1,1,0,0,0 };
                aCSMMassMarker.IxxIyyIzzIxyIxzIyzCM = aIxxIyyIzzIxyIxzIyzCM;

                //part marker
                CSMMarker cCSMMarker = (CSMMarker)aCSMPart.AddMarker();
                cCSMMarker.Name = "Mark2";

                double[] dAOriginMarkerMFf = { 1.0, 0.0, 0.0, 0.0, 
                               0.0, 1.0, 0.0, 0.0, 
                               0.0, 0.0, 1.0, 0.0, 
                               0.0, 0.0, 0.0, 1.0 };

                cCSMMarker.MFf = dAOriginMarkerMFf;
                
                //joint
                CSMRevoluteJoint aCSMRevoJoint = (CSMRevoluteJoint)aCSMAssembly.AddRevoluteJoint();
                aCSMRevoJoint.MarkerI = bCSMMarker.FullName();
                aCSMRevoJoint.MarkerJ = cCSMMarker.FullName();

                CSMRotationalMotion aCSMMotion = (CSMRotationalMotion)aCSMRevoJoint.AddRotation();
                aCSMMotion.Rotation = "2*pi*time";

                CSMSimulation aCSMSimulation = (CSMSimulation)aCSMAssembly.SimulationParameters();
                
                double Tstart = 0.0, Tend = 1.0, Hout = 0.1, Hmin = 1.0E-9, Hmax = 1.0, ErrorTolerance = 1.0E-6;
                aCSMSimulation.Tstart = Tstart;
                aCSMSimulation.Tend = Tend;
                aCSMSimulation.Hout = Hout;
                aCSMSimulation.Hmin = Hmin;
                aCSMSimulation.Hmax = Hmax;
                aCSMSimulation.ErrorTolerance = ErrorTolerance;

            }
            catch
            { 
            }

        }

        private void MOptionsTools_Click(object sender, EventArgs e)
        {
            ACMMainClass.aACMInterface.CreateRegistrationDialog();
        }

    }
}
