﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Colors;
using Autodesk.AutoCAD.GraphicsInterface;
using Autodesk.AutoCAD.BoundaryRepresentation;

namespace MotionAddon
{
    class ManipulateObject
    {
        public static void CreateSolid()
        {
            //Get the database
            Database Db = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Database;

            using (Transaction Trans = Db.TransactionManager.StartTransaction())
            {
                BlockTableRecord btr;
                BlockTable bt;
                bt = (BlockTable)Trans.GetObject(HostApplicationServices.WorkingDatabase.BlockTableId, OpenMode.ForRead);
                btr = (BlockTableRecord)Trans.GetObject(HostApplicationServices.WorkingDatabase.CurrentSpaceId, OpenMode.ForWrite);
                
                Solid3d NewSolid = new Solid3d();
                NewSolid.SetDatabaseDefaults();
                NewSolid.CreateBox(10, 10, 10);
                
                btr.AppendEntity(NewSolid); 

                Trans.AddNewlyCreatedDBObject(NewSolid, true);
                Trans.Commit();
            }
        }

        public static ObjectId CreateMarkerDefinition()
        {
            ObjectId newBtrId = new ObjectId(); 
            Database db = HostApplicationServices.WorkingDatabase;
            using (Transaction Trans = db.TransactionManager.StartTransaction()) 
            {
                BlockTable bt = (BlockTable)Trans.GetObject(db.BlockTableId, OpenMode.ForWrite);
                if ((bt.Has("Marker")))
                {
                    newBtrId = bt["Marker"];
                }
                else
                {
                    Line NewLineX = new Line(new Point3d(0, 0, 0), new Point3d(5, 0, 0));
                    Line NewLineY = new Line(new Point3d(0, 0, 0), new Point3d(0, 5, 0));
                    Line NewLineZ = new Line(new Point3d(0, 0, 0), new Point3d(0, 0, 5));
                    NewLineX.Color = Autodesk.AutoCAD.Colors.Color.FromColorIndex(ColorMethod.ByAci, 1);

                    NewLineY.Color = Autodesk.AutoCAD.Colors.Color.FromColorIndex(ColorMethod.ByAci, 3);

                    NewLineZ.Color = Autodesk.AutoCAD.Colors.Color.FromColorIndex(ColorMethod.ByAci, 5);

                    BlockTableRecord newBtr = new BlockTableRecord();
                    newBtr.Name = "Marker";
                    newBtrId = bt.Add(newBtr); 
                    Trans.AddNewlyCreatedDBObject(newBtr, true);

                    newBtr.AppendEntity(NewLineX);
                    newBtr.AppendEntity(NewLineY);
                    newBtr.AppendEntity(NewLineZ);

                    Trans.AddNewlyCreatedDBObject(NewLineX, true);
                    Trans.AddNewlyCreatedDBObject(NewLineY, true);
                    Trans.AddNewlyCreatedDBObject(NewLineZ, true);
                }

                Trans.Commit();
            }
            return newBtrId;
        }


        public static ObjectId CreateMarker(Point3d aPoint3d)
        {
            Database Db = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Database;
            BlockReference br;
            using (Transaction Trans = Db.TransactionManager.StartTransaction())
            {
                BlockTable bt = (BlockTable)(Trans.GetObject(Db.BlockTableId, OpenMode.ForWrite));
                BlockTableRecord btr = (BlockTableRecord)Trans.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite);
                br = new BlockReference(aPoint3d, CreateMarkerDefinition());
                btr.AppendEntity(br);
                Trans.AddNewlyCreatedDBObject(br, true);
                Trans.Commit();
            }
            return br.ObjectId;
        }
    }
}
